@@ -1,11 +1,10 @@
 package Net::IMAP::Simple;
-# $Id: Simple.pm,v 1.7 2005/04/28 12:00:00 cfaber Exp $
 use strict;
 use IO::File;
 use IO::Socket;
 
 use vars qw[$VERSION];
-$VERSION = '0.99';
+$VERSION = $1 if('$Id: Simple.pm,v 1.16_1 2006/10/02 11:16:37 cfaber Exp $' =~ /,v ([\d_.]+) /);
 
 =head1 NAME
 
@@ -22,9 +21,7 @@ Net::IMAP::Simple - Perl extension for simple IMAP account handling.
        die "Unable to connect to IMAP: $Net::IMAP::Simple::errstr\n";
 
     # Log on
-    my $msgs = $imap->login('user','pass');
-
-    if($msgs !~ /^\d+$/){
+    if(!$imap->login('user','pass')){
         print STDERR "Login failed: " . $imap->errstr . "\n";
         exit(64);
     }
@@ -32,7 +29,7 @@ Net::IMAP::Simple - Perl extension for simple IMAP account handling.
     # Print the subject's of all the messages in the INBOX
     my $nm = $imap->select('INBOX');
 
-    for(my $i = 1; $i < $nm; $i++){
+    for(my $i = 1; $i <= $nm; $i++){
         if($imap->seen($i)){
             print "*";
         } else {
@@ -50,18 +47,14 @@ Net::IMAP::Simple - Perl extension for simple IMAP account handling.
 
 This module is a simple way to access IMAP accounts.
 
-=head2 Methods
+=head1 OBJECT CREATION METHOD
 
 =over 4
 
 =item new
 
+ my $imap = Net::IMAP::Simple->new( $server [ :port ]); OR  my $imap = Net::IMAP::Simple->new( $server [, option_name => option_value ] );
 
-my $imap = Net::IMAP::Simple->new( $server [ :port ] ) 
-
- OR 
-
-my $imap = Net::IMAP::Simple->new( $server [, option_name => option_value ] );
 
 This class method constructs a new C<Net::IMAP::Simple> object. It takes one required parameter which is the server to connect to, and additional optional parameters.
 
@@ -69,14 +62,47 @@ The server parameter may specify just the server, or both the server and port nu
 
 On success an object is returned. On failure, nothing is returned and an error message is set to $Net::IMAP::Simple.
 
-Options:
+=head2 OPTIONS:
 
- port        => some port other than 143
- timeout     => connection timeout after (x) seconds
- retry       => try and reconnect (x) times
- retry_delay => wait (x) seconds before retrying
- use_v6      => 1|0 -- Use IPv6 sockets rather than IPv4
- bindaddr    => some local address to bind
+Options are provided as a hash to new()
+
+=item port => int
+
+Assign the port number (default: 143)
+
+=item timeout => int (default: 90)
+
+Connection timeout in seconds.
+
+=item retry => int (default: 1)
+
+Attempt to retry the connection attmpt (x) times before giving up
+
+=item retry_delay => int (default: 5)
+
+Wait (x) seconds before retrying a connection attempt
+
+=item use_v6 => BOOL
+
+If set to true, attempt to use IPv6 sockets rather than IPv4 sockets.
+
+This option requires the IO::Socket::INET6 module
+
+=item bindaddr => str
+
+Assign a local address to bind
+
+=item use_select_cache => BOOL
+
+Enable select() caching internally
+
+=item select_cache_ttl => int
+
+The number of seconds to allow a select cache result live before running $imap->select() again.
+
+=item debug => BOOL | \*HANDLE
+
+Enable debugging output. If \*HANDLE is a valid file handle, debugging will be written to it. Otherwise debugging will be written to STDOUT
 
 =cut
 
@@ -97,13 +123,22 @@ sub new {
     $self->{retry} = ($opts{retry} ? $opts{retry} : $self->_retry);
     $self->{retry_delay} = ($opts{retry_delay} ? $opts{retry_delay} : $self->_retry_delay);
     $self->{bindaddr} = $opts{bindaddr};
+    $self->{use_select_cache} = $opts{use_select_cache};
+    $self->{select_cache_ttl} = $opts{select_cache_ttl};
+    $self->{debug} = $opts{debug};
+    
 
+    # Pop the port off the address string if it's not an IPv6 IP address
+    if(!$self->{use_v6} && $self->{server} =~ /^[A-Fa-f0-9]{4}:[A-Fa-f0-9]{4}:/ && $self->{server} =~ s/:(\d+)$//g){
+        $self->{port} = $1;
+    }
+   
     my $c;
-    for(my $i = 0; $i < $self->{retry}; $i++){
+    for(my $i = 0; $i <= $self->{retry}; $i++){
 	if($self->{sock} = $self->_connect){
 		$c = 1;
 		last;
-	} else {
+	} elsif ($i < $self->{retry}) {
 		select(undef, undef, undef, $self->{retry_delay});
 	}
     }
@@ -133,7 +168,7 @@ sub _connect {
 	PeerPort => $self->{port},
 	Timeout  => $self->{timeout},
 	Proto    => 'tcp',
-	($self->{bindaddr} ? { LocalAddr => $self->{bindaddr} } : '')
+	($self->{bindaddr} ? { LocalAddr => $self->{bindaddr} } : ())
  );
 
  return $sock;
@@ -150,26 +185,27 @@ sub _sock_from   { $_[0]->{use_v6} ? 'IO::Socket::INET6' : 'IO::Socket::INET' }
 
 =pod
 
+=head1 METHODS
+
 =item login
 
   my $inbox_msgs = $imap->login($user, $passwd);
 
 This method takes two required parameters, a username and password. This pair
-is authenticated against the server. If authentication is successful the
-user's INBOX is selected.
+is authenticated against the server. If authentication is successful TRUE (1) will be returned
 
-On success, an integer which represents the number of messages in the INBOX is returned. Nothing is returned on failure and the errstr() error handler is set with the error message.
+Nothing is returned on failure and the errstr() error handler is set with the error message.
 
 =cut
 
 sub login {
-    my ( $self, $user, $pass ) = @_;
+ my ( $self, $user, $pass ) = @_;
 
-    $self->_process_cmd(
-        cmd     => [LOGIN => qq[$user "$pass"]],
-        final   => sub { $self->select('INBOX') },
-        process => sub { },
-    );
+ return $self->_process_cmd (
+	cmd     => [LOGIN => qq[$user "$pass"]],
+	final   => sub { 1 },
+	process => sub { },
+ );
 }
 
 =pod
@@ -183,14 +219,112 @@ Selects a folder named in the single required parameter. The number of messages
 =cut
 
 sub select {
-    my ( $self, $mbox ) = @_;
+ my ( $self, $mbox ) = @_;
 
-    my $messages;
-    $self->_process_cmd(
-        cmd     => [SELECT => _escape($mbox)],
-        final   => sub { $self->{last} = $messages },
-        process => sub { if ($_[0] =~ /^\*\s+(\d+)\s+EXISTS/i) { $messages = $1 } },
-    );
+ $mbox = 'INBOX' unless $mbox;
+
+ $self->{working_box} = $mbox;
+
+ if($self->{use_select_cache} && (time - $self->{BOXES}->{ $mbox }->{proc_time}) <= $self->{select_cache_ttl}){
+	return $self->{BOXES}->{$mbox}->{messages};
+ }
+
+ $self->{BOXES}->{$mbox}->{proc_time} = time;
+
+ my $t_mbox = $mbox;
+
+ $self->_process_cmd(
+	cmd     => [SELECT => _escape($t_mbox)],
+	final   => sub { $self->{last} = $self->{BOXES}->{$mbox}->{messages} },
+	process => sub {
+		if($_[0] =~ /^\*\s+(\d+)\s+EXISTS/i){
+			$self->{BOXES}->{$mbox}->{messages} = $1;
+		} elsif($_[0] =~ /^\*\s+FLAGS\s+\((.*?)\)/i){
+			$self->{BOXES}->{$mbox}->{flags} = [ split(/\s+/, $1) ];
+		} elsif($_[0] =~ /^\*\s+(\d+)\s+RECENT/i){
+			$self->{BOXES}->{$mbox}->{recent} = $1;
+		} elsif($_[0] =~ /^\*\s+OK\s+\[(.*?)\s+(.*?)\]/i){
+			my ($flag, $value) = ($1, $2);
+			if($value =~ /\((.*?)\)/){
+				$self->{BOXES}->{$mbox}->{sflags}->{$flag} = [split(/\s+/, $1)];
+			} else {
+				$self->{BOXES}->{$mbox}->{oflags}->{$flag} = $value;
+			}
+		}
+	},
+ ) || return;
+
+ return $self->{last}
+}
+
+=pod
+
+=item messages
+
+    print "Messages in Junk Mail -- " . $imap->messages("INBOX.Junk Mail") . "\n";
+
+This method is an alias for $imap->select
+
+=cut
+
+sub messages {
+ my ($self, $folder) = @_;
+ return $self->select($folder);
+}
+
+=pod
+
+=item flags
+
+    print "Avaliable server flags: " . join(", ", $imap->flags) . "\n";
+
+This method accepts an optional folder name and returns the current avaliable server flags as a list, for the selected folder. If no folder name is provided the last folder $imap->select'ed will be used.
+
+This method uses caching.
+
+=cut
+
+sub flags {
+ my ($self, $folder) = @_;
+
+ $self->select($folder);
+ return @{ $self->{BOXES}->{ $self->current_box }->{flags} };
+}
+
+
+=pod
+
+=item recent
+
+    print "Recent messages value: " . $imap->recent . "\n";
+
+This method accepts an optional folder name and returns the 'RECENT' value provided durning a SELECT result set. If no folder name is provided the last folder $imap->select'ed will be used.
+
+This method uses caching.
+
+=cut
+
+sub recent {
+ my ($self, $folder) = @_;
+
+ $self->select($folder);
+ return $self->{BOXES}->{ $self->current_box }->{recent};
+}
+
+
+=pod
+
+=item current_box
+
+   print "Current Mail Box folder: " . $imap->current_box . "\n";
+
+This method returns the current working mail box folder name.
+
+=cut
+
+sub current_box {
+ my ($self) = @_;
+ return ($self->{working_box} ? $self->{working_box} : 'INBOX');
 }
 
 =pod
@@ -290,7 +424,11 @@ sub get {
     $self->_process_cmd(
         cmd     => [FETCH => qq[$number rfc822]],
         final   => sub { pop @lines; \@lines },
-        process => sub { push @lines, $_[0] unless $_[0] =~ /^\*/ },
+        process => sub {
+		if($_[0] !~ /^\* \d+ FETCH/){
+			push @lines, join(' ', @_);
+		}
+	},
     );
 
 }
@@ -316,9 +454,11 @@ sub getfh {
         cmd     => [FETCH => qq[$number rfc822]],
         final   => sub { seek $file, 0, 0; $file },
         process => sub {
-                        defined($buffer) and print $file $buffer unless /^\*/;
+		if($_[0] !~ /^\* \d+ FETCH/){
+                        defined($buffer) and print $file $buffer;
                         $buffer = $_[0];
-                       },
+		}
+	},
     );
 }
 
@@ -326,19 +466,31 @@ sub getfh {
 
 =item quit
 
-  $item->quit;
+  $imap->quit;
+
+  OR
+
+  $imap->quit(BOOL);
 
 This method logs out of the IMAP server, expunges the selected mailbox,
 and closes the connection. No error message will ever be returned from this method.
 
+Optionally if BOOL is TRUE (1) then a hard quit is preformed which closes the socket connection. This hard quit will still issue both EXPUNGE and LOGOUT commands however the response is ignored and the socket is closed after issuing the commands.
+
 =cut
 
 sub quit {
-    my ( $self ) = @_;
-    $self->_send_cmd('EXPUNGE');
-    $self->_send_cmd('LOGOUT');
-    $self->_sock->close;
-    return 1;
+ my ( $self, $hq ) = @_;
+ $self->_send_cmd('EXPUNGE');
+
+ if(!$hq){   
+	$self->_process_cmd(cmd => ['LOGOUT'], final => sub {}, process => sub{});
+ } else {
+	$self->_send_cmd('LOGOUT');
+ }
+
+ $self->_sock->close;
+ return 1;
 }    
 
 =pod
@@ -378,15 +530,19 @@ sub delete {
 
 sub _process_list {
     my ($self, $line) = @_;
+    $self->_debug(caller, __LINE__, '_process_list', $line) if $self->{debug};
+
     my @list;
-    if ( $line =~ /^\*\s+LIST.*\s+\{\d+\}\s*$/i ) {
+    if ( $line =~ /^\*\s+(LIST|LSUB).*\s+\{\d+\}\s*$/i ) {
         chomp( my $res = $self->_sock->getline );
         $res =~ s/\r//;
         _escape($res);
         push @list, $res;
-    } elsif ( $line =~ /^\*\s+LIST.*\s+(\".*?\")\s*$/i ||
-              $line =~ /^\*\s+LIST.*\s+(\S+)\s*$/i ) {
-        push @list, $1;
+
+	$self->_debug(caller, __LINE__, '_process_list', $res) if $self->{debug};
+    } elsif ( $line =~ /^\*\s+(LIST|LSUB).*\s+(\".*?\")\s*$/i ||
+              $line =~ /^\*\s+(LIST|LSUB).*\s+(\S+)\s*$/i ) {
+        push @list, $2;
     }
     @list;
 }
@@ -402,7 +558,7 @@ sub _process_list {
 This method returns a list of mailboxes. When called with no arguments it
 recurses from the IMAP root to get all mailboxes. The first optional
 argument is a mailbox path and the second is the path reference. RFC 3501
-has more information.
+section 6.3.8 has more information.
 
 On failure nothing is returned and the errstr() error handler is set with the error message.
 
@@ -419,7 +575,7 @@ sub mailboxes {
         return $self->_process_cmd(
             cmd     => [LIST => qq[$ref *]],
             final   => sub { _unescape($_) for @list; @list },
-            process => sub { push @list, $self->_process_list($_[0]) },
+            process => sub { push @list, $self->_process_list($_[0]);},
         );
     } else {
         return $self->_process_cmd(
@@ -432,6 +588,45 @@ sub mailboxes {
 
 =pod
 
+=item mailboxes_subscribed
+
+  my @boxes   = $imap->mailboxes_subscribed;
+  my @folders = $imap->mailboxes_subscribed("Mail/%");
+  my @lists   = $imap->mailboxes_subscribed("lists/perl/*", "/Mail/");
+
+This method returns a list of mailboxes subscribed to. When called with no
+arguments it recurses from the IMAP root to get all mailboxes. The first optional
+argument is a mailbox path and the second is the path reference. RFC 3501
+has more information.
+
+On failure nothing is returned and the errstr() error handler is set with the error message.
+
+=cut
+
+sub mailboxes_subscribed {
+    my ( $self, $box, $ref ) = @_;
+    
+    $ref ||= '""';
+    my @list;
+    if ( ! defined $box ) {
+        # recurse, should probably follow
+        # RFC 2683: 3.2.2.  Subscriptions
+        return $self->_process_cmd(
+            cmd     => [LSUB => qq[$ref *]],
+            final   => sub { _unescape($_) for @list; @list },
+            process => sub { push @list, $self->_process_list($_[0]) },
+        );
+    } else {
+        return $self->_process_cmd(
+            cmd     => [LSUB => qq[$ref $box]],
+            final   => sub { _unescape($_) for @list; @list },
+            process => sub { push @list, $self->_process_list($_[0]) },
+        );
+    }
+}
+
+=pod
+
 =item create_mailbox
 
   print "Created" if $imap->create_mailbox( "/Mail/lists/perl/advocacy" );
@@ -464,14 +659,14 @@ the required argument. Returns true on success, false on failure and the errstr(
 =cut
 
 sub expunge_mailbox {
-    my ( $self, $box ) = @_;
-    _escape( $box );
-    
-    return $self->_process_cmd(
-        cmd     => [EXPUNGE => $box],
-        final   => sub { 1 },
-        process => sub { },
-    );
+ my ($self, $box) = @_;
+ return if !$self->select($box);
+
+ return $self->_process_cmd(
+	cmd     => ['EXPUNGE'],
+	final   => sub { 1 },
+	process => sub { },
+ );
 }
 
 =pod
@@ -522,6 +717,52 @@ sub rename_mailbox {
 
 =pod
 
+=item folder_subscribe
+
+  print "Subscribed" if $imap->folder_subscribe( "/Mail/lists/perl/advocacy" );
+
+This method subscribes to the folder. Returns true on success, false on failure
+and the errstr() error handler is set with the error message.
+
+=cut
+
+sub folder_subscribe {
+ my ($self, $box) = @_;
+ $self->select($box); # XXX does it matter if this fails?
+ _escape($box);
+ 
+ return $self->_process_cmd(
+        cmd     => [SUBSCRIBE => $box],
+        final   => sub { 1 },
+        process => sub { },
+ );
+}
+
+=pod
+
+=item folder_unsubscribe
+
+  print "Unsubscribed" if $imap->folder_unsubscribe( "/Mail/lists/perl/advocacy" );
+
+This method unsubscribes to the folder. Returns true on success, false on failure
+and the errstr() error handler is set with the error message.
+
+=cut
+
+sub folder_unsubscribe {
+ my ($self, $box) = @_;
+ $self->select($box);
+ _escape($box);
+ 
+ return $self->_process_cmd(
+        cmd     => [UNSUBSCRIBE => $box],
+        final   => sub { 1 },
+        process => sub { },
+ );
+}
+
+=pod
+
 =item copy
 
   print "copied" if $imap->copy( $message_number, $mailbox );
@@ -543,9 +784,11 @@ sub copy {
     );
 }
 
+=pod
+
 =item errstr
 
- print "Login ERROR: " . $imap->errstr . "\n" if($imap->login($user, $pass) !~ /^\d+$/);
+ print "Login ERROR: " . $imap->errstr . "\n" if !$imap->login($user, $pass);
 
 Return the last error string captured for the last operation which failed.
 
@@ -574,7 +817,10 @@ sub _send_cmd {
     my ( $self, $name, $value ) = @_;
     my $sock = $self->_sock;
     my $id   = $self->_nextid;
-    my $cmd  = qq[$id $name $value\r\n];
+    my $cmd  = "$id $name" . ($value ? " $value" : "") . "\r\n";
+
+    $self->_debug(caller, __LINE__, '_send_cmd', $cmd) if $self->{debug};
+
     { local $\; print $sock $cmd; }
     return ($sock => $id);
 }
@@ -583,17 +829,35 @@ sub _cmd_ok {
     my ($self, $res) = @_;
     my $id = $self->_count;
 
+    $self->_debug(caller, __LINE__, '_send_cmd', $res) if $self->{debug};
+
     if($res =~ /^$id\s+OK/i){
 	return 1;
     } elsif($res =~ /^$id\s+(?:NO|BAD)(?:\s+(.+))?/i){
 	$self->_seterrstr($1 || 'unknown error');
 	return 0;
     } else {
-	$self->_seterrstr("unknown return string: $res");
+	$self->_seterrstr("warning unknown return string: $res");
 	return;
     }
+}
 
-    return;
+sub _read_multiline {
+  my ($self, $sock, $count) = @_;
+  
+  my @lines;
+  my $read_so_far = 0;
+  while ($read_so_far < $count) {
+    push @lines, $sock->getline;
+    $read_so_far += length($lines[-1]);
+  }
+  if($self->{debug}){
+	for(my $i = 0; $i < @lines; $i++){
+		$self->_debug(caller, __LINE__, '_read_multiline', "[$i] $lines[$i]");
+	}
+  }
+
+  return @lines;
 }
 
 sub _process_cmd {
@@ -602,13 +866,20 @@ sub _process_cmd {
 
     my $res;
     while ( $res = $sock->getline ) {
-        my $ok = $self->_cmd_ok($res);
-	if ( defined($ok) && $ok == 1 ) {
-            return $args{final}->($res);
-        } elsif ( defined($ok) && ! $ok ) {
-            return;
-        } else {
+	$self->_debug(caller, __LINE__, '_process_cmd', $res) if $self->{debug};
+
+        if ( $res =~ /^\*.*\{(\d+)\}$/ ) {
             $args{process}->($res);
+            $args{process}->($_) foreach $self->_read_multiline($sock, $1);
+        } else {
+            my $ok = $self->_cmd_ok($res);
+   	    if ( defined($ok) && $ok == 1 ) {
+                return $args{final}->($res);
+            } elsif ( defined($ok) && ! $ok ) {
+                return;
+            } else {
+		$args{process}->($res);
+            }
         }
     }
 }
@@ -616,9 +887,24 @@ sub _process_cmd {
 sub _seterrstr {
  my ($self, $err) = @_;
  $self->{_errstr} = $err;
+ $self->_debug(caller, __LINE__, '_seterrstr', $err) if $self->{debug};
  return;
 }
 
+sub _debug {
+ my ($self, $package, $filename, $line, $dline, $routine, $str) = @_;
+
+ $str =~ s/\n/\\n/g;
+ $str =~ s/\r/\\r/g;
+ $str =~ s/\cM/^M/g;
+
+ my $line = "[$package :: $filename :: $line\@$dline -> $routine] $str\n";
+ if(ref($self->{debug}) eq 'GLOB'){
+	write($self->{debug}, $line);
+ } else {
+	print STDOUT $line;
+ }
+}
 
 =pod
 
